@@ -1,6 +1,6 @@
 /* lsmbox -- list nr of total and read messages for one or several mailboxes
  *
- * Copyright (C) 2002-2004 David Weinehall
+ * Copyright (C) 2002-2006 David Weinehall
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,9 +14,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software Foundation,
- *  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
-
 #include "lsmbox.h"
 
 extern int optind;
@@ -41,6 +40,7 @@ static int newonly = 0;
 static int nototal = 0;
 static int noold = 0;
 static int quit = 0;
+static int padding = -1;
 
 static time_t mailcheck = 0;
 static time_t selectedtimer = 0;
@@ -88,6 +88,9 @@ static void usage(void)
 		  "  -o, --no-old          do not display the number of old "
 		  "unread messages\n"
 		  "                          each mailbox contains\n"
+		  "  -p, --padding         padding for the mailbox column;\n"
+		  "                          leave empty to adjust to "
+		  "smallest possible\n"
 		  "  -s, --no-summary      do not display a summary\n"
 		  "  -S, --short-names     only show the name of the mailbox\n"
 		  "                          rather than the entire path\n"
@@ -111,7 +114,7 @@ static void version(void)
 		PACKAGE_VERSION,
 		_("Written by David Weinehall.\n"
 		  "\n"
-		  "Copyright (C) 2002-2004 David Weinehall (tao@acc.umu.se)\n"
+		  "Copyright (C) 2002-2006 David Weinehall\n"
 		  "This is free software; see the source for copying "
 		  "conditions.  There is NO\n"
 		  "warranty; not even for MERCHANTABILITY or FITNESS FOR "
@@ -133,7 +136,7 @@ static void reset_curses(void)
 /**
  * Setup curses
  *
- *  @return 1 on success, 0 on failure
+ * @return 1 on success, 0 on failure
  */
 static int init_curses(void)
 {
@@ -167,7 +170,7 @@ static int init_curses(void)
 /**
  * Setup locales
  *
- *  @return 0 on success, errno on failure
+ * @return 0 on success, errno on failure
  */
 static int init_locales(const char *name)
 {
@@ -302,9 +305,28 @@ static int curses_getch(void)
 #endif /* USE_CURSES */
 }
 
+/**
+ * Local version of basename
+ *
+ * @param str A pointer to the basename part of the string
+ * @return basename of string
+ */
+char *basenamep(const char *const str)
+{
+	char *tmp = strrchr(str, '/');
+
+	if (!tmp)
+		tmp = (char *)str;
+	else
+		tmp++;
+
+	return tmp;
+}
+
 /**
  * Check for keypresses and mouse-events
- *  @return 1 on event, 0 otherwise
+ *
+ * @return 1 on event, 0 otherwise
  */
 static int eventcheck(void)
 {
@@ -356,8 +378,8 @@ static int eventcheck(void)
 /**
  * Output a formatted string
  *
- *   @param fmt The formatting string
- *   @param args The rest of the arguments
+ * @param fmt The formatting string
+ * @param args The rest of the arguments
  */
 static void output_string(const char *fmt, ...)
 {
@@ -375,11 +397,26 @@ static void output_string(const char *fmt, ...)
 	va_end(args);
 }
 
+/**
+ * Calculate padsize
+ *
+ * @param str The string to output
+ * @param maxlen The maxlen of the line
+ */
+void output_string_padded(const char *const str, int maxlen)
+{
+	char *fmt;
+
+	assert(asprintf(&fmt, "%%-%ds  ", maxlen));
+	output_string(fmt, str);
+	free(fmt);
+}
+
 /**
  * Wrapper for strdup that checks for NULL
  *
- *   @param str A pointer to the string to duplicate
- *  @return NULL if str is NULL, the duplicated string otherwise
+ * @param str A pointer to the string to duplicate
+ * @return NULL if str is NULL, the duplicated string otherwise
  */
 static char *strdup_null(const char *str)
 {
@@ -389,9 +426,9 @@ static char *strdup_null(const char *str)
 /**
  * Copy a string and return a pointer to its end
  *
- *   @param dst A pointer to the destination
- *   @param src A pointer to the source
- *  @return A pointer to the end of the destination string
+ * @param dst A pointer to the destination
+ * @param src A pointer to the source
+ * @return A pointer to the end of the destination string
  */
 static char *internal_stpcpy(char *dst, const char *src)
 {
@@ -408,9 +445,9 @@ static char *internal_stpcpy(char *dst, const char *src)
  * Join a variable amount of strings,
  * with an optional delimiter between each
  *
- *   @param delimiter The string delimiter
- *   @param ... The strings to join
- *  @return A newly allocated string with the result
+ * @param delimiter The string delimiter
+ * @param ... The strings to join
+ * @return A newly allocated string with the result
  */
 static char *strjoin(const char *delimiter, ...)
 {
@@ -460,11 +497,11 @@ static char *strjoin(const char *delimiter, ...)
  * Split a string into an array, using the specified delimiter
  * to tell where to make the splits
  *
- *   @param str A pointer to the string to split
- *   @param delimiter A pointer to the delimiter
- *   @param max_tokens The maximum number of tokens to split the string into
- *                     into; 0 means that the string should be fully split
- *  @return A NULL-terminated, newly allocated array of strings
+ * @param str A pointer to the string to split
+ * @param delimiter A pointer to the delimiter
+ * @param max_tokens The maximum number of tokens to split the string into
+ *                   into; 0 means that the string should be fully split
+ * @return A NULL-terminated, newly allocated array of strings
  */
 static char **strsplit(const char *str, const char *delimiter,
 		       const int max_tokens)
@@ -516,11 +553,11 @@ static char **strsplit(const char *str, const char *delimiter,
  *  0 if num1 == num2,
  *  1 if num1 > num2
  *
- *   @param num1 A pointer to the first number
- *   @param num2 A pointer to the second number
- *  @return -1 if num1 < num2
- *           0 if num1 == num2
- *           1 if num1 > num2
+ * @param num1 A pointer to the first number
+ * @param num2 A pointer to the second number
+ * @return -1 if num1 < num2
+ *          0 if num1 == num2
+ *          1 if num1 > num2
  */
 static int ulongcmp(const void *num1, const void *num2)
 {
@@ -536,8 +573,8 @@ static int ulongcmp(const void *num1, const void *num2)
 /**
  * Skip all leading whitespace in a string
  *
- *   @param str The string to process
- *  @return A new pointer into the string with leading whitespace skipped
+ * @param str The string to process
+ * @return A new pointer into the string with leading whitespace skipped
  */
 static char *skip_whitespace(char *str)
 {
@@ -551,7 +588,7 @@ static char *skip_whitespace(char *str)
 /**
  * Chop off all trailing whitespace in a string
  *
- *   @param str The string to process
+ * @param str The string to process
  */
 static void chop_whitespace(char *str)
 {
@@ -565,10 +602,10 @@ static void chop_whitespace(char *str)
 /**
  * Skip past a configuration-option to its value
  *
- *   @param str The string to process
- *   @param skip SKIP_EQ to skip `='
- *		 SKIP_QU to skip `"' and `''
- *  @return A new pointer to the value of the configuration-option
+ * @param str The string to process
+ * @param skip SKIP_EQ to skip `='
+ *	       SKIP_QU to skip `"' and `''
+ * @return A new pointer to the value of the configuration-option
  */
 static char *skip_config_option(char *str, int skip)
 {
@@ -590,7 +627,7 @@ static char *skip_config_option(char *str, int skip)
 /**
  * Get the user-name of the caller of the program
  *
- *  @return A pointer to the user-name
+ * @return A pointer to the user-name
  */
 static const char *get_user_name(void)
 {
@@ -604,7 +641,7 @@ static const char *get_user_name(void)
 /**
  * Free a NULL-terminated array of strings, and the array itself
  *
- *   @param array The array to free
+ * @param array The array to free
  */
 static void strfreev(char **array)
 {
@@ -622,9 +659,9 @@ static void strfreev(char **array)
 /**
  * Expand a path-name
  *
- *   @param str The path to expand
- *  @return A pointer to the string with the new path;
- *          needs to be freed after use
+ * @param str The path to expand
+ * @return A pointer to the string with the new path;
+ *         needs to be freed after use
  */
 static char *path_expand(const char *str)
 {
@@ -668,8 +705,8 @@ static char *path_expand(const char *str)
 /**
  * Parse the configuration-file
  *
- *   @param fname The name of the configuration-file to parse
- *  @return zero on success, non-zero on failure
+ * @param fname The name of the configuration-file to parse
+ * @return zero on success, non-zero on failure
  */
 static int parse_config_file(const char *fname)
 {
@@ -779,6 +816,24 @@ static int parse_config_file(const char *fname)
 					free(mailcommand);
 					mailcommand = strdup(str);
 				}
+			} else if (!strncmp(str, PADDING_LEN_STR,
+					    strlen(PADDING_LEN_STR))) {
+				str = skip_config_option(str, SKIP_EQ);
+
+				if (!str) {
+					error = E_SYNTAX;
+				} else {
+					unsigned long tmp;
+
+					tmp = strtoul(str, NULL, 10);
+
+					if ((tmp == LONG_MAX &&
+					     errno == ERANGE) ||
+					    (tmp > PADDING_MAX))
+						error = E_SYNTAX;
+					else
+						padding = (int)tmp;
+				}
 			} else {
 				error = E_UNKNOWN;
 			}
@@ -794,16 +849,19 @@ static int parse_config_file(const char *fname)
 				free(mailfolderpath);
 				mailfolderpath = NULL;
 			} else if (!strncmp(str, MARK_OLD_STR,
-				strlen(MARK_OLD_STR))) {
+					    strlen(MARK_OLD_STR))) {
 				markold = 0;
 			} else if (!strncmp(str, MH_SEQ_UNSEEN_STR,
-				strlen(MH_SEQ_UNSEEN_STR))) {
+					    strlen(MH_SEQ_UNSEEN_STR))) {
 				free(unseenstr);
 				unseenstr = strdup(UNSEEN_STR);
 			} else if (!strncmp(str, MAIL_COMMAND_STR,
-				strlen(MAIL_COMMAND_STR))) {
+					    strlen(MAIL_COMMAND_STR))) {
 				free(mailcommand);
 				mailcommand = NULL;
+			} else if (!strncmp(str, PADDING_LEN_STR,
+					    strlen(PADDING_LEN_STR))) {
+				padding = -1;
 			} else {
 				error = E_UNKNOWN;
 			}
@@ -865,8 +923,8 @@ EXIT:
 /**
  * Parse a mailbox in Maildir format
  *
- *   @param mbox A mailstat structure for the mailbox
- *  @return zero on success, non-zero on failure
+ * @param mbox A mailstat structure for the mailbox
+ * @return zero on success, non-zero on failure
  */
 static int parse_maildir(struct mailstat *mbox)
 {
@@ -989,8 +1047,8 @@ EXIT:
 /**
  * Parse a mailbox in MH format
  *
- *   @param mbox A mailstat structure for the mailbox
- *  @return zero on success, non-zero on failure
+ * @param mbox A mailstat structure for the mailbox
+ * @return zero on success, non-zero on failure
  */
 static int parse_mhdir(struct mailstat *mbox)
 {
@@ -1218,8 +1276,8 @@ EXIT:
 /**
  * Parse a mailbox in mbox/MMDF format
  *
- *   @param mbox A mailstat structure for the mailbox
- *  @return zero on success, non-zero on failure
+ * @param mbox A mailstat structure for the mailbox
+ * @return zero on success, non-zero on failure
  */
 static int parse_mailbox(struct mailstat *mbox)
 {
@@ -1309,7 +1367,7 @@ EXIT:
 /**
  * Update statistics for all mailboxes in the mboxes array
  *
- *  @return 0 on success, errno on failure
+ * @return 0 on success, errno on failure
  */
 static int update_mailbox_stats(void)
 {
@@ -1488,18 +1546,16 @@ static int update_mailbox_stats(void)
 		selected = i - 1;
 
 EXIT:
-
 	return errno;
 }
 
 /**
  * Show statistics for all mailboxes in the mboxes array
  *
- *   @param now the time show_mailbox_stats was called
- *
- *  @return 0 on success, errno on failure
+ * @param now the time show_mailbox_stats was called
+ * @return 0 on success, errno on failure
  */
-static int show_mailbox_stats(const time_t now)
+static int show_mailbox_stats(const time_t now, const int width)
 {
 	int ypos = 0;
 	int i;
@@ -1517,9 +1573,9 @@ static int show_mailbox_stats(const time_t now)
 	totals.old = 0;
 
 	bold_on();
-	output_string(_(
-		"mailbox:                                     "
-		"%s%s:%s\n"),
+	output_string_padded(_("mailbox:"), width);
+
+	output_string("%s%s:%s\n",
 		nototal ? "" : _("total:      "),
 		(markold || noold) ? _("new") : _("unread"),
 		(markold && !noold) ? _("        old:") : "");
@@ -1583,18 +1639,14 @@ static int show_mailbox_stats(const time_t now)
 			totals.old += mboxes[i].old;
 		}
 
-		if (!(tmp = strrchr(mboxes[i].name, '/')))
-			tmp = mboxes[i].name;
-		else
-			tmp++;
+		tmp = basenamep(mboxes[i].name);
 
 		modtime = mboxes[i].atime > mboxes[i].mtime ? mboxes[i].atime :
 							      mboxes[i].mtime;
 
 		hilight_on(now - modtime <= HILIGHT_TIME, (selected == i));
 
-		output_string("%-43s  ",
-			      shortnames ? tmp : mboxes[i].name);
+		output_string_padded(shortnames ? tmp : mboxes[i].name, width);
 
 		if (!nototal)
 			output_string("%-10lu  ", mboxes[i].total);
@@ -1611,9 +1663,8 @@ static int show_mailbox_stats(const time_t now)
 	}
 
 	if (!nosummary) {
-		output_string(_(
-			      "\nSummary:              "
-			      "                       "));
+		output_string("\n");
+		output_string_padded(_("Summary:"), width);
 
 		if (!nototal)
 			output_string("%-10lu  ", totals.total);
@@ -1629,16 +1680,15 @@ static int show_mailbox_stats(const time_t now)
 	update_screen();
 
 EXIT:
-
 	return errno;
 }
 
 /**
  * The program's main-function
  *
- *   @param argc The number of arguments
- *   @param argv The arguments
- *  @return 0 on success, errno on failure
+ * @param argc The number of arguments
+ * @param argv The arguments
+ * @return 0 on success, errno on failure
  */
 int main(int argc, char *argv[])
 {
@@ -1646,6 +1696,7 @@ int main(int argc, char *argv[])
 	int opt_index;
 	size_t i, start, end;
 	time_t oldtime, now;
+	int maxlen = 0;
 
 	char **list = NULL;
 	char *userrcpath = NULL;
@@ -1656,6 +1707,16 @@ int main(int argc, char *argv[])
 	if (init_locales(PACKAGE_NAME))
 		goto EXIT;
 
+	homepath = strdup_null(getenv("HOME"));
+	unseenstr = strdup(UNSEEN_STR);
+
+	if (homepath) {
+		userrcpath = strjoin(NULL, homepath, "/.lsmboxrc", NULL);
+
+		if (parse_config_file(userrcpath))
+			goto EXIT;
+	}
+
 	/* Parse the command-line options */
 	while ((optc = getopt_long(argc, argv, OPTSTR,
 				   options, &opt_index)) != -1) {
@@ -1683,6 +1744,21 @@ int main(int argc, char *argv[])
 			noold = 1;
 			break;
 
+		case 'p':
+			if (!optarg || !strlen(optarg)) {
+				padding = 0;
+				break;
+			}
+
+			tmp = strtoul(optarg, NULL, 10);
+
+			if ((tmp == LONG_MAX && errno == ERANGE) ||
+			    (tmp > PADDING_MAX))
+				goto EXIT;
+
+			padding = (int)tmp;
+			break;
+
 		case 's':
 			nosummary = 1;
 			break;
@@ -1710,16 +1786,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	homepath = strdup_null(getenv("HOME"));
-	unseenstr = strdup(UNSEEN_STR);
-
-	if (homepath) {
-		userrcpath = strjoin(NULL, homepath, "/.lsmboxrc", NULL);
-
-		if (parse_config_file(userrcpath))
-			goto EXIT;
-	}
-
 	if (!spoolfilepath) {
 		if (!(spoolfilepath = strdup_null(getenv("MAIL")))) {
 			spoolfilepath = strdup_null(SPOOLFILE_PATH);
@@ -1793,9 +1859,32 @@ int main(int argc, char *argv[])
 
 	assert(mboxes = calloc(end - start + 1, sizeof (struct mailstat)));
 
-	for (i = start; i < end; i++)
+	for (i = start; i < end; i++) {
+		int len;
+
 		mboxes[i - start].name = path_expand(list[i]);
 
+		if (shortnames)
+			len = strlen(basenamep(mboxes[i - start].name));
+		else
+			len = strlen(mboxes[i - start].name);
+
+		if (len > maxlen)
+			maxlen = len;
+	}
+
+	if (strlen(_("mailbox:")) > maxlen)
+		maxlen = strlen(_("mailbox:"));
+
+	if (strlen(_("Summary:")) > maxlen)
+		maxlen = strlen(_("Summary:"));
+
+	/* old behaviour */
+	if (padding == -1)
+		maxlen = 43;
+	else if ((padding >= 0) && (padding > maxlen))
+		maxlen = padding;
+
 	oldtime = 0;
 	now = 0;
 
@@ -1806,7 +1895,8 @@ int main(int argc, char *argv[])
 		/* Update mailstats every mailcheck seconds */
 		if ((eventcheck() && !quit) || now - oldtime > mailcheck) {
 			update_mailbox_stats();
-			show_mailbox_stats(now);
+
+			show_mailbox_stats(now, maxlen);
 
 			if (time(&oldtime) == (time_t)-1)
 				break;
@@ -1815,6 +1905,8 @@ int main(int argc, char *argv[])
 		/* Check for a keypress every second */
 		if (!quit)
 			(void)sleep((unsigned int)1);
+		if (errno == EINTR)
+			errno = 0;
 	} while (!errno && continuous && !quit);
 
 	if (continuous)
